@@ -3,6 +3,8 @@
 
   Copyright (C) 1999-2000 by Matthias Kiefer
                             <matthias.kiefer@gmx.de>
+		2001	  by Stanislav Visnovsky
+			    <visnovsky@kde.org>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -22,11 +24,6 @@
 #ifndef CATALOG_H
 #define CATALOG_H
 
-
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
 #include <qintcache.h>
 #include <qobject.h>
 #include <qlist.h>
@@ -98,8 +95,11 @@ public:
    enum IOStatus{OK,OS_ERROR,NO_PERMISSIONS,RECOVERED_PARSE_ERROR,PARSE_ERROR,NO_FILE};
    enum DiffResult{DiffOk, DiffNotFound, DiffNeedList};
 
-   static IOStatus info(const QString url,PoInfo& info);
+   static IOStatus info(const QString url,PoInfo& info, QStringList &wordList, bool updateWordList);
    static PoInfo headerInfo(const CatalogItem);
+   static bool findInFile(const QString url, FindOptions options );
+   
+   static bool stopStaticRead;
 
    /**
    * reads header information from the file and searches for charset
@@ -440,6 +440,9 @@ private:
    QValueList<CatalogItem> _entries;
    /** The header of the po-file. */
    CatalogItem _header;
+   
+   /** list of comments with obsolete entries */
+   QValueList<QString> _obsolete;
 
    bool _modified;
    bool _readOnly;
@@ -448,14 +451,14 @@ private:
    QValueList<uint> _untransIndex;
    QValueList<uint> _errorIndex;
 
-   QList<CatalogView> _views;
+   QPtrList<CatalogView> _views;
 
    SaveSettings _saveSettings;
    IdentitySettings _identitySettings;
    MiscSettings _miscSettings;
 
-   QList<EditCommand> _undoList;
-   QList<EditCommand> _redoList;
+   QPtrList<EditCommand> _undoList;
+   QPtrList<EditCommand> _redoList;
 
    QTextCodec *fileCodec;
 
