@@ -22,17 +22,14 @@
 #ifndef CATALOGSETTINGS_H
 #define CATALOGSETTINGS_H
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
 #include <qstring.h>
 #include <qregexp.h>
+#include <qdatetime.h>
 
 class QTextCodec;
+class QStringList;
 
 enum FileEncoding{Locale,UTF8,UTF16};
-enum DateFormat{Default,Local,Custom};
 
 
 struct SaveSettings
@@ -47,7 +44,7 @@ struct SaveSettings
     FileEncoding encoding;
     bool useOldEncoding;
 
-    DateFormat dateFormat;
+    Qt::DateFormat dateFormat;
     QString customDateFormat;
 
     bool autoSyntaxCheck;
@@ -92,6 +89,14 @@ struct MiscSettings
    QRegExp singularPlural;
 };
 
+struct TagSettings
+{
+    /**
+    * A list of regular expressions defining tags
+    */
+    QStringList expressions;
+};
+
 QString charsetString(FileEncoding);
 QString charsetString(QTextCodec *codec);
 
@@ -114,39 +119,47 @@ namespace Defaults
        static const FileEncoding encoding=UTF8;
        static const bool useOldEncoding=true;
 
-       static const DateFormat dateFormat=Default;
-       static const QString customDateFormat;
+       static const Qt::DateFormat dateFormat=Qt::ISODate;
 
        static const bool autoSyntaxCheck=true;
+
+       static const QString customDateFormat();
    };
 
    class Identity
    {
     public:
-       static QString authorName();
-       static QString authorEmail();
-       static QString languageName();
-       static QString languageCode();
-       static QString mailingList();
-       static QString timezone();
+       Identity();
+       QString authorName();
+       QString authorEmail();
+       QString languageName();
+       QString languageCode();
+       QString mailingList();
+       QString timezone();
 
-       static const int numberOfPluralForms;
+       const int numberOfPluralForms;
 
     private:
-       static QString _authorName;
-       static QString _authorEmail;
-       static QString _mailingList;
-       static QString _timezone;
-       static QString _languageName;
-       static QString _languageCode;
+       QString _authorName;
+       QString _authorEmail;
+       QString _mailingList;
+       QString _timezone;
+       QString _languageName;
+       QString _languageCode;
    };
 
    class Misc
    {
       public:
-         static const QChar accelMarker;
-         static const QRegExp contextInfo;
-         static const QRegExp singularPlural;
+         QChar accelMarker();
+         QRegExp contextInfo();
+         QRegExp singularPlural();
+   };
+   
+   class Tag
+   {
+      public:
+	 static QStringList expressions();
    };
 }
 
