@@ -29,7 +29,7 @@
 
 #include <klocale.h>
 #include <kiconloader.h>
-#include <kapp.h>
+#include <kapplication.h>
 
 
 #define PAGE_IDENTITY 0
@@ -38,13 +38,18 @@
 #define PAGE_SPELL    3
 #define PAGE_SEARCH   4
 #define PAGE_DIFF     5
-#define PAGE_CATMAN   6
-#define PAGE_MISC     7
+#define PAGE_MISC     6
+
+
+// FIXME: remove this (and set correct number in define) when the catalog manager options can be set in kBabel again 
+#define PAGE_CATMAN   7
+QString a1( I18N_NOOP("Catalog Manager"));
+QString a2( I18N_NOOP("Options for the Catalog Manager"));
 
 KBabelPreferences::KBabelPreferences(SearchSettings searchOpts
                         , EditorSettings editOpts ,SaveSettings saveOpts
                         , IdentitySettings idOpts,CatManSettings catManOpts
-                        , MiscSettings miscOpts,QList<ModuleInfo> ml)
+                        , MiscSettings miscOpts,QPtrList<ModuleInfo> ml)
     : KDialogBase(IconList, i18n("Preferences"),
                   Help|Default|Ok|Apply|Cancel, Ok,0,0,false)
 {
@@ -88,12 +93,6 @@ KBabelPreferences::KBabelPreferences(SearchSettings searchOpts
     _diffPage = new DiffPreferences(frame);
     _diffPage->setSettings(editOpts);
  
-    
-    frame = addVBoxPage(i18n("Catalog Manager"),i18n("Options for the Catalog Manager"),BarIcon("catalogmanager",32));
-    _catManPage = new CatManPreferences(frame);
-    _catManPage->setSettings(catManOpts);
-
-
     frame = addVBoxPage(i18n("misc settings","Miscellaneous")
                 ,i18n("Miscellaneous Settings"),DesktopIcon("misc"));
     _miscPage = new MiscPreferences(frame);
@@ -125,8 +124,8 @@ void KBabelPreferences::updateSaveSettings(SaveSettings settings)
 
 void KBabelPreferences::updateCatManSettings(CatManSettings settings)
 {
-    _catManPage->setSettings(settings);
-    _catManSettings=settings;
+//    _catManPage->setSettings(settings);
+//    _catManSettings=settings;
 }
 
 void KBabelPreferences::updateSearchSettings(SearchSettings settings)
@@ -165,7 +164,7 @@ void KBabelPreferences::slotDefault()
          _searchPage->defaults();
          break;
       case PAGE_CATMAN:
-         _catManPage->defaults();
+//         _catManPage->defaults();
          break;
 	  case PAGE_MISC:
 		 _miscPage->defaults();
@@ -202,7 +201,7 @@ void KBabelPreferences::slotApply()
          emit searchOptionsChanged(_searchSettings);
          break;
       case PAGE_CATMAN:
-         _catManPage->mergeSettings(_catManSettings);
+//         _catManPage->mergeSettings(_catManSettings);
          emit catManOptionsChanged(_catManSettings);
          break;
 	  case PAGE_MISC:
@@ -231,7 +230,7 @@ void KBabelPreferences::slotOk()
    _spellPage->mergeSettings(_editorSettings);
    _diffPage->mergeSettings(_editorSettings);
    _searchPage->mergeSettings(_searchSettings);
-   _catManPage->mergeSettings(_catManSettings);
+//   _catManPage->mergeSettings(_catManSettings);
    _miscPage->mergeSettings(_miscSettings);
 
    emit identityOptionsChanged(_idSettings);
@@ -252,7 +251,7 @@ void KBabelPreferences::slotCancel()
     _spellPage->setSettings(_editorSettings);
     _diffPage->setSettings(_editorSettings);
     _searchPage->setSettings(_searchSettings);
-    _catManPage->setSettings(_catManSettings);
+//    _catManPage->setSettings(_catManSettings);
 	_miscPage->setSettings(_miscSettings);
 
     QDialog::reject();
