@@ -43,6 +43,7 @@ class KColorButton;
 class KComboBox;
 class KSpellConfig;
 class KURLRequester;
+class QPushButton;
 
 struct ModuleInfo;
 
@@ -156,7 +157,7 @@ class SearchPreferences : public QWidget
 {
     Q_OBJECT
 public:
-    SearchPreferences(QList<ModuleInfo> ml,QWidget *parent = 0);
+    SearchPreferences(QPtrList<ModuleInfo> ml,QWidget *parent = 0);
     virtual ~SearchPreferences();
     void mergeSettings(SearchSettings& settings);
     void setSettings(const SearchSettings&);
@@ -166,29 +167,7 @@ private:
     QCheckBox* _searchButton;
     KComboBox* _defaultModuleBox;
 
-    QList<ModuleInfo> moduleList;
-};
-
-class CatManPreferences : public KTabCtl
-{
-    Q_OBJECT
-public:
-    CatManPreferences(QWidget *parent = 0);
-    virtual ~CatManPreferences();
-
-    void mergeSettings(CatManSettings& settings) const;
-    void setSettings(const CatManSettings&);
-    void defaults();
-
-private:
-    KURLRequester* _poDirEdit;
-    KURLRequester* _potDirEdit;
-
-    QCheckBox* _openWindowButton;
-
-    CmdEdit* _dirCmdEdit;
-    CmdEdit* _fileCmdEdit;
-    QCheckBox* _killButton;
+    QPtrList<ModuleInfo> moduleList;
 };
 
 class MiscPreferences : public QWidget
@@ -202,12 +181,18 @@ public:
    void setSettings(const MiscSettings&);
    void defaults();
 
+private slots:
+   void regExpButtonClicked();
+   
 private:
    void setContextInfo(QString reg);
    QString contextInfo() const;
    
    KLineEdit *contextInfoEdit;
    KLineEdit *accelMarkerEdit;
+   
+   QDialog *_regExpEditDialog;
+   QPushButton *_regExpButton;
 };
 
 class DiffPreferences : public QWidget
