@@ -27,7 +27,7 @@
 #include "roughtransdlg.h"
 #include "tagextractor.h"
 
-#include <qarray.h>
+#include <qmemarray.h>
 #include <qcheckbox.h>
 #include <qhbuttongroup.h>
 #include <qlabel.h>
@@ -37,7 +37,7 @@
 #include <qvbox.h>
 #include <qwhatsthis.h>
 
-#include <kapp.h>
+#include <kapplication.h>
 #include <kconfig.h>
 #include <kglobal.h>
 #include <klocale.h>
@@ -82,10 +82,12 @@ RoughTransDlg::RoughTransDlg(KBabelDictBox *dict, Catalog *cat
     QHButtonGroup *bBox = new QHButtonGroup(box);
     bBox->setMargin(0);
     bBox->setFrameStyle(QFrame::NoFrame);
-    allButton = new QRadioButton(i18n("All &entries"),bBox);
-    untransButton = new QRadioButton(i18n("U&ntranslated and fuzzy entries"),bBox);
+    whatBox = bBox;
+    untransButton = new QCheckBox(i18n("U&ntranslated entries"),bBox);
+    fuzzyButton = new QCheckBox(i18n("&Fuzzy entries"),bBox);
+    transButton = new QCheckBox(i18n("T&ranslated entries"),bBox);
 
-    connect(bBox,SIGNAL(clicked(int)),this,SLOT(msgButtonClicked()));
+    connect(bBox,SIGNAL(clicked(int)),this,SLOT(msgButtonClicked(int)));
 
     QWhatsThis::add(bBox,i18n("<qt><p><b>What entries to translate</b></p>"
                 "<p>Choose here, for which entries of the file KBabel "
@@ -95,35 +97,40 @@ RoughTransDlg::RoughTransDlg(KBabelDictBox *dict, Catalog *cat
     bBox = new QHButtonGroup(box);
     bBox->setFrameStyle(QFrame::NoFrame);
     bBox->setMargin(0);
-    wholeMsgButton = new QRadioButton(i18n("&Only whole message"),bBox);
-    singleWordButton = new QRadioButton(i18n("A&llow single word translation")
+
+    fuzzyMatchButton = new QCheckBox(i18n("All&ow fuzzy translation (slow)")
+            ,bBox);
+    singleWordButton = new QCheckBox(i18n("A&llow single word translation")
             ,bBox);
 
     QWhatsThis::add(bBox,i18n("<qt><p><b>How messages get translated</b></p>"
                 "<p>Here you can define if a message can only get translated "
-                "completely or if KBabel is supposed to try translating "
+                "completely, if similar messages are acceptable or if KBabel "
+                "is supposed to try translating "
                 "the single words of a message if no translation of the "
-                "complete message was found.</p></qt>"));
+                "complete message or similar message was found.</p></qt>"));
+ 
 
     box = new QVGroupBox(i18n("Options"),configWidget);
-    
+
     markFuzzyButton = new QCheckBox(i18n("&Mark changed entries as fuzzy"),box);
     markFuzzyButton->setChecked(true);
     QWhatsThis::add(markFuzzyButton,
             i18n("<qt><p><b>Mark changed entries as fuzzy</b></p>"
-          "<p>When an translation for a message was found, the entry "
-          "will marked <b>fuzzy</b> by default. This is, because the "
+          "<p>When a translation for a message is found, the entry "
+          "will be marked <b>fuzzy</b> by default. This is because the "
           "translation is just guessed by KBabel and you should always "
-          "check the results carefully. Deactivate this option only, if "
+          "check the results carefully. Deactivate this option only if "
           "you know what you are doing.</p></qt>"));
 
+
     connect(markFuzzyButton, SIGNAL(toggled(bool))
             , this, SLOT(fuzzyButtonToggled(bool)));
 
     QVGroupBox *dBox = new QVGroupBox(i18n("Dictionaries"),configWidget);
     configWidget->setStretchFactor(dBox,1);
 
-    QList<ModuleInfo> moduleList = dict->moduleInfos();
+    QPtrList<ModuleInfo> moduleList = dict->moduleInfos();
 
     KConfig *config = KGlobal::config();
     KConfigGroupSaver gs(config,"RoughTranslation");
@@ -140,37 +147,27 @@ RoughTransDlg::RoughTransDlg(KBabelDictBox *dict, Catalog *cat
     dictChooser = new DictChooser(moduleList,selectedList,dBox,"dictChooser");
 
     QWhatsThis::add(dictChooser,i18n("<qt><p><b>Dictionaries</b></p>"
-                "<p>Choose here, what dictionaries has to be used for "
+                "<p>Choose here, which dictionaries have to be used for "
                 "finding a translation. If you select more than one "
                 "dictionary, they are used in the same order as they "
                 "are displayed in the list.</p></qt>"));
- 
+
     progressbar = new KProgress(mw,"progressbar");
     progressbar->setTextEnabled(true);
     progressbar->setFormat("%v/%m (%p%)");
     mainLayout->addWidget(progressbar);
 
-    bool flag = config->readBoolEntry("AllMessages",true);
-    if(flag)
-    {
-        allButton->setChecked(true);
-    }
-    else
-    {
-        untransButton->setChecked(true);
-    }
+    transButton->setChecked(config->readBoolEntry("Translated",false));
+    untransButton->setChecked(config->readBoolEntry("Untranslated",true));
+    fuzzyButton->setChecked(config->readBoolEntry("Fuzzies",false));
 
-    flag = config->readBoolEntry("WholeMessage",true);
-    if(flag)
-    {
-        wholeMsgButton->setChecked(true);
-    }
-    else
-    {
-        singleWordButton->setChecked(true);
-    }
+    bool flag = config->readBoolEntry("fuzzyMatch",true);
+    fuzzyMatchButton->setChecked(flag);
 
-    msgButtonClicked();
+    flag = config->readBoolEntry("singleWord",true);
+    singleWordButton->setChecked(flag);
+
+    msgButtonClicked(0);
 }
 
 RoughTransDlg::~RoughTransDlg()
@@ -190,46 +187,83 @@ void RoughTransDlg::slotUser1()
     cancel=false;
 
     bool markFuzzy = markFuzzyButton->isChecked();
+    bool translated = transButton->isChecked();
+    bool untranslated = untransButton->isChecked();
+    bool fuzzies = fuzzyButton->isChecked();
 
-    int total;
-    QArray<int> indizes;
+    int total=0;
+    QMemArray<int> indizes;
     int index=0;
-    if(allButton->isChecked())
+    if(translated)
     {
-        total = catalog->numberOfEntries();
-        indizes.resize(total);
-        for(index = 0; index < total; index++)
-        {
-            indizes[index]=index;
-        }
+        total = catalog->numberOfEntries()-catalog->numberOfFuzzies()-
+            catalog->numberOfUntranslated();
     }
-    else
+    if(untranslated)
+    {
+        total += catalog->numberOfUntranslated();
+    }
+    if(fuzzies)
+    {
+        total += catalog->numberOfFuzzies();
+    }
+        
+    indizes.resize(total);
+    int counter=0;
+    if( (fuzzies && catalog->isFuzzy(0)) 
+         || (untranslated && catalog->isUntranslated(0)) 
+         || (translated && !catalog->isFuzzy(0) && !catalog->isUntranslated(0)))
     {
-        total = catalog->numberOfUntranslated()+catalog->numberOfFuzzies();
-        indizes.resize(total);
-        int counter=0;
-        if(catalog->isFuzzy(0) || catalog->isUntranslated(0))
-        {
             indizes[0]=index;
             counter++;
-        }
-        while(index >= 0 && counter < total)
+    }
+    while(index >= 0 && counter < total)
+    {
+        int fuzzyIndex=-1;
+        if(fuzzies)
         {
-            int fuzzyIndex=catalog->nextFuzzy(index);
-            int untransIndex=catalog->nextUntranslated(index);
+            fuzzyIndex=catalog->nextFuzzy(index);
+        }
+        int untransIndex=-1;
+        if(untranslated)
+        {
+            untransIndex=catalog->nextUntranslated(index);
+        }
 
-            if(fuzzyIndex<0)
-                fuzzyIndex=untransIndex;
-            if(untransIndex<0)
-                untransIndex=fuzzyIndex;
+        if(fuzzyIndex<0)
+             fuzzyIndex=untransIndex;
+        if(untransIndex<0)
+             untransIndex=fuzzyIndex;
 
-            index=(fuzzyIndex < untransIndex)? fuzzyIndex : untransIndex;
-
-            if(index>=0)
+        int newIndex=(fuzzyIndex < untransIndex)? fuzzyIndex : untransIndex;
+        int transIndex=index+1;
+        if(translated)
+        {
+            while(transIndex < catalog->numberOfEntries() 
+                    && (catalog->isUntranslated(transIndex) 
+                           || catalog->isFuzzy(transIndex)))
             {
-                indizes[counter]=index;
-                counter++;
+                transIndex++;
             }
+            if(transIndex >= catalog->numberOfEntries())
+                transIndex = -1;
+
+            if(transIndex < 0)
+                transIndex = newIndex;
+            if(newIndex < 0)
+                newIndex = transIndex;
+        
+            index=(newIndex < transIndex)? newIndex : transIndex;
+        }
+        else
+        {
+            index = newIndex;
+        }
+
+        if(index>=0)
+        {
+            indizes[counter]=index;
+            counter++;
         }
     }
 
@@ -243,6 +277,7 @@ void RoughTransDlg::slotUser1()
     exactTransCounter=0;
     partTransCounter=0;
     bool singleWords=singleWordButton->isChecked();
+    bool fuzzyMatch=fuzzyMatchButton->isChecked();
     QRegExp contextReg=catalog->miscSettings().contextInfo;
     QChar accelMarker=catalog->miscSettings().accelMarker;
     QRegExp endPunctReg("[\\.?!: ]+$");
@@ -255,8 +290,8 @@ void RoughTransDlg::slotUser1()
             break;
         }
         
-        progressbar->setValue(i+1);
-        kapp->processEvents();
+        progressbar->setProgress(i+1);
+        kapp->processEvents(100);
      
         QString msg=catalog->msgid(indizes[i]);
         QString translation;
@@ -276,31 +311,57 @@ void RoughTransDlg::slotUser1()
             msg.replace(contextReg,"");
         }
 
+	// try exact translation
+	QStringList::Iterator dit = dictList.begin();
+	while(translation.isEmpty() && dit != dictList.end())
+	  {
+	    dictBox->setActiveModule(*dit);
+	    translation = dictBox->translate(msg);
 
-        QStringList::Iterator dit = dictList.begin();
-        while(translation.isEmpty() && dit != dictList.end())
-        {
-            dictBox->setActiveModule(*dit);
-            translation = dictBox->translate(msg);
-            
-            ++dit;
-        }
+	    ++dit;
+	  }
 
-        if(!translation.isEmpty())
-        {
-            exactTransCounter++;
-        }
-        else if(singleWords)
-        {
+	if(!translation.isEmpty())
+	  {
+	    exactTransCounter++;
+	  }
+
+	// try fuzzy translation
+	else if (fuzzyMatch) { 
+	  QString tr;
+	  int score, best_score = 0;
+	  dit = dictList.begin();
+	  while(dit != dictList.end())
+	    {
+	      dictBox->setActiveModule(*dit);
+	      tr = dictBox->fuzzyTranslation(msg,score);
+
+	      if (score > best_score) {
+		translation = tr;
+		best_score = score;
+	      }
+	  
+	      ++dit;
+	    }
+
+	  if(!translation.isEmpty())
+	    {
+	      partTransCounter++;
+	    } 
+	}
+
+	// try single word translation
+        if(translation.isEmpty() && singleWords)
+	  {
             QStringList wordList;
             QChar accel;
             QString endingPunctuation;
             int pos = msg.findRev(endPunctReg);
             if(pos >= 0)
-            {
+	      {
                 endingPunctuation = msg.right(msg.length()-pos);
-            }
-             
+	      }
+	    
             msg=msg.simplifyWhiteSpace();
             msg=msg.stripWhiteSpace();
 
@@ -328,7 +389,7 @@ void RoughTransDlg::slotUser1()
                         {
                             if(!word.contains(digitReg))
                                 wordList.append(word);
-                                
+
                             word=QString::null;
                         }
                     }
@@ -473,10 +534,10 @@ void RoughTransDlg::slotUser1()
 
     int nothing=totalTried-partTransCounter-exactTransCounter;
     KLocale *locale = KGlobal::locale();
-    QString statMsg = i18n("Result of the translation:\n\n"
+    QString statMsg = i18n("Result of the translation:\n"
             "Edited entries: %1\n"
             "Exact translations: %2 (%3%)\n"
-            "Non exact translations: %4 (%5%)\n"
+            "Approximate translations: %4 (%5%)\n"
             "Nothing found: %6 (%7%)")
             .arg( locale->formatNumber(totalTried,0) )
             .arg( locale->formatNumber(exactTransCounter,0) )
@@ -497,9 +558,9 @@ void RoughTransDlg::slotUser1()
             KMessageBox::information(this, statMsg
                     , i18n("Rough translation statistics"));
 
-            msgButtonClicked();
+            msgButtonClicked(0);
         }
-        progressbar->setValue(0);
+        progressbar->setProgress(0);
         configWidget->setEnabled(true);
         active = false;
         stop=false;
@@ -520,11 +581,17 @@ void RoughTransDlg::slotUser1()
     KConfigGroupSaver gs(config,"RoughTranslation");
     config->writeEntry("Selected",dictChooser->selectedDicts());
 
-    bool flag=allButton->isChecked();
-    config->writeEntry("AllMessages",flag);
-    flag=wholeMsgButton->isChecked();
-    config->writeEntry("WholeMessage",flag);
-    
+    bool flag=transButton->isChecked();
+    config->writeEntry("Translated",flag);
+    flag=untransButton->isChecked();
+    config->writeEntry("Untranslated",flag);
+    flag=fuzzyButton->isChecked();
+    config->writeEntry("Fuzzies",flag);
+    flag=singleWordButton->isChecked();
+    config->writeEntry("singleWord",flag);
+    flag=fuzzyMatchButton->isChecked();
+    config->writeEntry("fuzzyMatch",flag);
+
     accept();
 }
 
@@ -551,19 +618,43 @@ void RoughTransDlg::slotUser3()
     cancel=true;
 }
 
-void RoughTransDlg::msgButtonClicked()
+void RoughTransDlg::msgButtonClicked(int id)
 {
-    int total;
-    if(allButton->isChecked())
+    if(!transButton->isChecked() && !untransButton->isChecked() 
+            && !fuzzyButton->isChecked())
     {
-        total = catalog->numberOfEntries();
-    }
-    else
-    {
-        total = catalog->numberOfUntranslated()+catalog->numberOfFuzzies();
+        QButton *button = whatBox->find(id);
+        if(button == transButton)
+        {
+            transButton->setChecked(true);
+        }
+        else if(button == untransButton)
+        {
+            untransButton->setChecked(true);
+        }
+        else if(button == fuzzyButton)
+        {
+            fuzzyButton->setChecked(true);
+        }
     }
     
-    progressbar->setRange(0,total);
+
+    int total=0; 
+    if(transButton->isChecked())
+    {
+        total = catalog->numberOfEntries()-catalog->numberOfFuzzies()
+            -catalog->numberOfUntranslated();
+    }
+    if(untransButton->isChecked())
+    {
+        total += catalog->numberOfUntranslated();
+    }
+    if(fuzzyButton->isChecked())
+    {
+        total += catalog->numberOfFuzzies();
+    }
+
+    progressbar->setTotalSteps(total);
 
     enableButton(User1,total>0);
 }
@@ -573,12 +664,12 @@ void RoughTransDlg::fuzzyButtonToggled(bool on)
     if(!on)
     {
         QString msg=i18n("<qt><p>"
-          "When an translation for a message was found, the entry "
-          "will marked <b>fuzzy</b> by default. This is, because the "
+          "When a translation for a message is found, the entry "
+          "will be marked <b>fuzzy</b> by default. This is because the "
           "translation is just guessed by KBabel and you should always "
-          "check the results carefully. Deactivate this option only, if "
+          "check the results carefully. Deactivate this option only if "
           "you know what you are doing.</p></qt>");
-          
+
         KMessageBox::information(this, msg, QString::null,"MarkFuzzyWarningInRoughTransDlg");
     }
 }
