@@ -17,7 +17,7 @@
 
 */
 
-#include <qmultilineedit.h>
+#include <qtextedit.h>
 #include <qprogressdialog.h>
 
 #include <qinputdialog.h>
@@ -26,9 +26,9 @@
 #include <kmessagebox.h>
 #include <kfiledialog.h>
 #include <kurlrequester.h>
-#include <kstddirs.h>
+#include <kstandarddirs.h>
 
-#include "kapp.h"
+#include "kapplication.h"
 #include "KDBSearchEngine.h"
 
 #include "dbscan.h"
@@ -49,7 +49,7 @@
 #include <qcheckbox.h>
 #include <qspinbox.h>
 #include <qslider.h>
-#include <qarray.h>
+#include <qmemarray.h>
 
 #include "database.h"
 
@@ -334,7 +334,7 @@ bool KDBSearchEngine::messagesForPackage(const QString& package
       count++;	       
       if(count%step==0){
            emit progress(100*count/totalRecord);
-           kapp->processEvents();
+           kapp->processEvents(100);
          }
       if(stopNow) {
 	stopNow=false;
@@ -419,7 +419,7 @@ void KDBSearchEngine::repeat()
   
    progressStarts(i18n("Searching repeated string"));
    
-   static QMultiLineEdit *mle=new QMultiLineEdit();
+   static QTextEdit *mle=new QTextEdit();
    mle->clear();
    
    bool inlibs;
@@ -429,7 +429,7 @@ void KDBSearchEngine::repeat()
       count++;	       
       if(count%step==0){
            emit progress(100*count/totalRecord);
-           kapp->processEvents();
+           kapp->processEvents(100);
          }
       if(stopNow) {
 	stopNow=false;
@@ -476,7 +476,7 @@ bool KDBSearchEngine::startSearchNow(int searchmode)
   stopNow=false; // Remove dirty.
   clearResults();
 
-
+ 
   if(searching )
   {
    hasError(i18n("Another search has already been started"));
@@ -556,7 +556,7 @@ bool KDBSearchEngine::startSearchNow(int searchmode)
   timeval now;
   gettimeofday(&now,NULL);
   //fprintf(stderr,"\n%ld.%ld\n",now.tv_sec,now.tv_usec);
-  double tim=1.0*now.tv_usec/1000000.0+now.tv_sec;
+  //double tim=1.0*now.tv_usec/1000000.0+now.tv_sec;
   int pos;
 
 
@@ -585,8 +585,8 @@ bool KDBSearchEngine::startSearchNow(int searchmode)
 
 //   Emit progress, process event and check stop now
       if(count%step==0){
-           emit progress(100*count/totalprogress);
-           kapp->processEvents();
+           emit progress(100*count/QMAX(totalprogress,1));
+           kapp->processEvents(100);
          }
       if(stopNow) {
 	stopNow=false;
@@ -668,7 +668,7 @@ bool KDBSearchEngine::startSearchNow(int searchmode)
 	    if  (!foundExact && (rules & RegExp ))
 		{
 	          QRegExp reg(*idMod);
-	          regexp=(reg.match(msgId)!=-1);
+	          regexp=(reg.search(msgId)!=-1);
 		} else regexp=false;
    	  nn++;
 	  } while(intra && nn<item.numTra);
@@ -775,8 +775,8 @@ bool KDBSearchEngine::startSearchNow(int searchmode)
  */
 
 bool KDBSearchEngine::startSingleSearch(QString searchString,
-				     unsigned int pattern1Limit=0,unsigned int pattern2Limit=0,
-				     bool inTranslation=false)
+				     unsigned int pattern1Limit,unsigned int /*pattern2Limit*/,
+				     bool inTranslation)
 {
   /*
     Check Ret 
@@ -791,7 +791,7 @@ bool KDBSearchEngine::startSingleSearch(QString searchString,
 
 	 
   QRegExp  reg("[a-zA-Z0-9_%"/*+remchar*/+regaddchar+"]+");
-  while((in=reg.match(searchString,in+len,&len))!=-1) {nw++;}
+  while((in=reg.search(searchString,in+len))!=-1) {nw++;len = reg.matchedLength();}
   in=0;
   len=0;
  // fprintf(stderr,"asas %d\n",nw);
@@ -804,7 +804,8 @@ bool KDBSearchEngine::startSingleSearch(QString searchString,
   if((nw<pattern1Limit) && (nw >1))
     for(unsigned int k=0;k<nw;k++)
       {
-	in=reg.match(searchString,in+len,&len);
+	in=reg.search(searchString,in+len);
+	len = reg.matchedLength();
 	QString regToAdd = searchString;
 	regToAdd.replace(in,len,"[a-zA-Z0-9_%"+regaddchar+"]*");
 	regToAdd.append("$");
@@ -827,7 +828,7 @@ if(inTranslation)
   Start a search for a list of string
  */
  
-//bool KDBSearchEngine::startListSearch(QList<QString> searchStrList)
+//bool KDBSearchEngine::startListSearch(QPtrList<QString> searchStrList)
 //{
   //  searchStringList=searchStrList;
 //  return startSearchNow();
@@ -909,7 +910,8 @@ void  KDBSearchEngine::readSettings(KConfigBase *config)
 
  QString defaultLang;
  QString oldLang=lang;
- defaultLang = Defaults::Identity::languageCode();
+ Defaults::Identity def;
+ defaultLang = def.languageCode();
  lang=config->readEntry("Language",defaultLang);
 
  QString defaultDir;
@@ -1302,10 +1304,10 @@ progressStarts(i18n("Searching words"));
 QValueList<QString>::Iterator wlit;
 wordlist=dm->wordsIn(phrase);
 int nw=wordlist.count();
-//QArray<WordItem> wi(nw);
-QArray<uint32> numofloc(nw),currentloc(nw);
-QArray<int> score(nw);
-QArray<uint32 *> loc(nw),locorig(nw);
+//QMemArray<WordItem> wi(nw);
+QMemArray<uint32> numofloc(nw),currentloc(nw);
+QMemArray<int> score(nw);
+QMemArray<uint32 *> loc(nw),locorig(nw);
 QValueList<uint32> resloc;
 QValueList<int> resfound;
 QValueList<KeyAndScore> keylist;
@@ -1382,7 +1384,7 @@ if(min!=BIGNUMBER)
 	if(count%step==0)
 	{
            emit progress(100*count/totalprogress);
-            kapp->processEvents();
+            kapp->processEvents(100);
           }
 	
 	currentloc[j]++;
@@ -1406,7 +1408,7 @@ if(found>=thrs)
   int in=0,len=0;
   QString keyst=dm->getKey(min);
   QRegExp  reg("[a-zA-Z0-9_%"/*+remchar*/+regaddchar+"]+");
-  while((in=reg.match(keyst,in+len,&len))!=-1) {nword++;}
+  while((in=reg.search(keyst,in+len))!=-1) {nword++; len=reg.matchedLength();}
 
   if(found>=nword*thresholdorig/100)  //
    {
@@ -1455,7 +1457,7 @@ for(int j=0;j<nres;j++)
     int in=0,len=0,nword=0;
     int remove=retnot?common:0;
      QRegExp  reg("[a-zA-Z0-9_%"/*+remchar*/+regaddchar+"]+");
-     while((in=reg.match(strkey,in+len,&len))!=-1) {nword++;}
+     while((in=reg.search(strkey,in+len))!=-1) {nword++;len=reg.matchedLength();}
 
 //    kdDebug(0) << nword << "NWORD  " << resfound[j] << "FOUND  " 
 //    << resfound[j]-remove << "REAL  " << remove << "to be remove   " << endl; 
