@@ -42,8 +42,8 @@ DbSeFactory::~DbSeFactory()
 }
 
 
-QObject *DbSeFactory::create( QObject *parent, const char *name
-				, const char *classname, const QStringList &)
+QObject *DbSeFactory::createObject( QObject *parent, const char *name,
+				    const char *classname, const QStringList &)
 {
 	if(QCString(classname) != "SearchEngine")
 	{
