@@ -31,7 +31,7 @@ class QTimer;
 class KConfig;
 
 class Catalog;
-class PreferencesWidget;
+class AuxiliaryPreferencesWidget;
 
 class PoAuxiliary : public SearchEngine
 {
@@ -83,7 +83,7 @@ protected slots:
     void loadAuxiliary();
     
 private:
-    QGuardedPtr<PreferencesWidget> prefWidget;
+    QGuardedPtr<AuxiliaryPreferencesWidget> prefWidget;
     Catalog *catalog;
     QString auxPackage;
     QString auxTranslator;
