@@ -24,7 +24,7 @@
 #include <catalog.h>
 #include <tagextractor.h>
 
-#include <kapp.h>
+#include <kapplication.h>
 #include <kdebug.h>
 #include <klocale.h>
 
@@ -93,7 +93,7 @@ bool CompendiumData::load(KURL url)
         if( (100*(i+1))%total < 100 )
         {
             emit progress((100*(i+1))/total);
-            kapp->processEvents();
+            kapp->processEvents(100);
         }
 
         QString temp = _catalog->msgid(i);
@@ -218,3 +218,4 @@ QStringList CompendiumData::wordList(const QString string)
     return QStringList::split(' ',result);
 }
 
+#include "compendiumdata.moc"
